/*
 * Copyright Bigeon Emmanuel (2014)
 *
 * emmanuel@bigeon.fr
 *
 * This software is a computer program whose purpose is to
 * provide a generic framework for console applications.
 *
 * This software is governed by the CeCILL license under French law and
 * abiding by the rules of distribution of free software.  You can  use,
 * modify and/or redistribute the software under the terms of the CeCILL
 * license as circulated by CEA, CNRS and INRIA at the following URL
 * "http://www.cecill.info".
 *
 * As a counterpart to the access to the source code and  rights to copy,
 * modify and redistribute granted by the license, users are provided only
 * with a limited warranty  and the software's author,  the holder of the
 * economic rights,  and the successive licensors  have only  limited
 * liability.
 *
 * In this respect, the user's attention is drawn to the risks associated
 * with loading,  using,  modifying and/or developing or reproducing the
 * software by the user in light of its specific status of free software,
 * that may mean  that it is complicated to manipulate,  and  that  also
 * therefore means  that it is reserved for developers  and  experienced
 * professionals having in-depth computer knowledge. Users are therefore
 * encouraged to load and test the software's suitability as regards their
 * requirements in conditions enabling the security of their systems and/or
 * data to be ensured and,  more generally, to use and operate it in the
 * same conditions as regards security.
 *
 * The fact that you are presently reading this means that you have had
 * knowledge of the CeCILL license and that you accept its terms.
 */
/**
 * gclc:fr.bigeon.gclc.command.CommandParametersTest.java
 * Created on: Nov 18, 2016
 */
package fr.bigeon.gclc.command;

import static org.junit.Assert.assertEquals;
import static org.junit.Assert.assertFalse;
import static org.junit.Assert.assertNull;
import static org.junit.Assert.assertTrue;

import java.util.HashSet;
import java.util.Set;

import org.junit.Test;

/**
 * <p>
 * TODO
 *
 * @author Emmanuel Bigeon
 *
 */
public class CommandParametersTest {

    /**
     * Test method for {@link fr.bigeon.gclc.command.CommandParameters#CommandParameters(java.util.Set, java.util.Set, boolean)}.
     */
    @Test
    public final void testCommandParameters(){
        Set<String> strings = new HashSet<>();
        Set<String> bools = new HashSet<>();
        CommandParameters parameters = new CommandParameters(bools, strings,
                true);

        assertFalse(parameters.parseArgs("-ungivenFlag"));
        parameters = new CommandParameters(bools, strings, false);

        assertTrue(parameters.parseArgs("-ungivenFlag"));
    }

    /**
     * Test method for {@link fr.bigeon.gclc.command.CommandParameters#get(java.lang.String)}.
     */
    @Test
    public final void testGet(){
        Set<String> strings = new HashSet<>();
        Set<String> bools = new HashSet<>();

        bools.add("boolFlag");
        strings.add("str");

        CommandParameters parameters = new CommandParameters(bools, strings,
                true);

        assertNull(parameters.get("ungiven"));
        assertNull(parameters.get("str"));

        parameters.parseArgs("-ungiven", "val");
        assertNull(parameters.get("ungiven"));
        assertNull(parameters.get("str"));

        parameters.parseArgs("-str", "val");
        assertNull(parameters.get("ungiven"));
        assertEquals("val", parameters.get("str"));

        parameters.parseArgs("-ungiven");
        assertNull(parameters.get("ungiven"));
        assertEquals("val", parameters.get("str"));
    }

    /**
     * Test method for {@link fr.bigeon.gclc.command.CommandParameters#getAdditionals()}.
     */
    @Test
    public final void testGetAdditionals(){
        Set<String> strings = new HashSet<>();
        Set<String> bools = new HashSet<>();

        bools.add("boolFlag");
        strings.add("str");

        CommandParameters parameters = new CommandParameters(bools, strings,
                true);

        parameters.parseArgs("-boolFlag");
        assertTrue(parameters.getAdditionals().isEmpty());

        parameters.parseArgs("-ungiven");
        assertTrue(parameters.getAdditionals().isEmpty());

        parameters = new CommandParameters(bools, strings, false);

        parameters.parseArgs("-boolFlag");
        assertTrue(parameters.getAdditionals().isEmpty());

        parameters.parseArgs("-ungiven");
        assertTrue(parameters.getAdditionals().contains("ungiven"));
        assertEquals(1, parameters.getAdditionals().size());
    }

    /**
     * Test method for {@link fr.bigeon.gclc.command.CommandParameters#getBool(java.lang.String)}.
     */
    @Test
    public final void testGetBool(){
        Set<String> strings = new HashSet<>();
        Set<String> bools = new HashSet<>();

        bools.add("boolFlag");
        strings.add("str");

        CommandParameters parameters = new CommandParameters(bools, strings,
                true);

        assertFalse(parameters.getBool("ungiven"));
        assertFalse(parameters.getBool("boolFlag"));

        parameters.parseArgs("-boolFlag");
        assertTrue(parameters.getBool("boolFlag"));
        assertFalse(parameters.getBool("ungiven"));

        parameters.parseArgs("-ungiven");
        assertFalse(parameters.getBool("ungiven"));
        assertTrue(parameters.getBool("boolFlag"));

        parameters = new CommandParameters(bools, strings, false);

        assertFalse(parameters.getBool("ungiven"));
        assertFalse(parameters.getBool("boolFlag"));

        parameters.parseArgs("-boolFlag");
        assertTrue(parameters.getBool("boolFlag"));
        assertFalse(parameters.getBool("ungiven"));

        parameters.parseArgs("-ungiven");
        assertFalse(parameters.getBool("ungiven"));
        assertTrue(parameters.getBool("boolFlag"));
    }

    /**
     * Test method for {@link fr.bigeon.gclc.command.CommandParameters#parseArgs(java.lang.String[])}.
     */
    @Test
    public final void testParseArgs(){
        Set<String> strings = new HashSet<>();
        Set<String> bools = new HashSet<>();

        bools.add("boolFlag");
        strings.add("str");

        CommandParameters parameters = new CommandParameters(bools, strings,
                true);

        assertFalse(parameters.parseArgs("-ungivenFlag"));
        assertFalse(parameters.parseArgs("-str"));
        assertTrue(parameters.parseArgs("-boolFlag"));
        assertTrue(parameters.parseArgs("-str", "-boolFlag"));
        assertTrue(parameters.parseArgs("-boolFlag", "-str", "val"));

    }

    /**
     * Test method for {@link fr.bigeon.gclc.command.CommandParameters#set(java.lang.String, boolean)}.
     */
    @Test
    public final void testSetStringBoolean(){
        Set<String> strings = new HashSet<>();
        Set<String> bools = new HashSet<>();

        bools.add("boolFlag");
        strings.add("str");

        CommandParameters parameters = new CommandParameters(bools, strings,
                true);

        assertFalse(parameters.getBool("ungiven"));
        assertFalse(parameters.getBool("boolFlag"));

        parameters.set("boolFlag", true);
        assertTrue(parameters.getBool("boolFlag"));
        assertFalse(parameters.getBool("ungiven"));

        parameters.set("ungiven", true);
        assertFalse(parameters.getBool("ungiven"));
        assertTrue(parameters.getBool("boolFlag"));

        parameters = new CommandParameters(bools, strings, false);

        assertFalse(parameters.getBool("ungiven"));
        assertFalse(parameters.getBool("boolFlag"));

        parameters.set("boolFlag", true);
        assertTrue(parameters.getBool("boolFlag"));
        assertFalse(parameters.getBool("ungiven"));

        parameters.set("ungiven", true);
        assertFalse(parameters.getBool("ungiven"));
        assertTrue(parameters.getBool("boolFlag"));
    }

    /**
     * Test method for {@link fr.bigeon.gclc.command.CommandParameters#set(java.lang.String, java.lang.String)}.
     */
    @Test
    public final void testSetStringString(){
        Set<String> strings = new HashSet<>();
        Set<String> bools = new HashSet<>();

        bools.add("boolFlag");
        strings.add("str");

        CommandParameters parameters = new CommandParameters(bools, strings,
                true);

        assertNull(parameters.get("ungiven"));
        assertNull(parameters.get("str"));

        parameters.set("ungiven", "val");
        assertNull(parameters.get("ungiven"));
        assertNull(parameters.get("str"));

        parameters.set("str", "val");
        assertNull(parameters.get("ungiven"));
        assertEquals("val", parameters.get("str"));

        parameters.set("ungiven", "val");
        assertNull(parameters.get("ungiven"));
        assertEquals("val", parameters.get("str"));

        parameters = new CommandParameters(bools, strings, false);

        assertNull(parameters.get("ungiven"));
        assertNull(parameters.get("str"));

        parameters.set("ungiven", "val");
        assertNull(parameters.get("ungiven"));
        assertNull(parameters.get("str"));

        parameters.set("str", "val");
        assertNull(parameters.get("ungiven"));
        assertEquals("val", parameters.get("str"));

        parameters.set("ungiven", "val");
        assertNull(parameters.get("ungiven"));
        assertEquals("val", parameters.get("str"));
    }

}
